function batchresults = batchanalysis_Tutorial6()

%Batchanalysis script written by Brad Wyble in March 2014,
%customized for data files from Tutorial 6 blockfile; a Stroop paradigm.


%Chooses the correct data directory based on whether one is using Mac or PC
if IsOSX
    datadir='data/';
else
    datadir='data\';
end
sca


%Creates the set of subject numbers's data files you wish to use
subnum_list = {'2' '3' '4' '5'};
numsubs = length(subnum_list);


%Sets the conditions of your factors:                       
%Factor 1
types_of_trials = 3;  %The number of types of trials in this experiment (congruent, incongruent, and neutral)


%Other factors:



%Create the sets of "buckets" in which we'll accumulate data
%batchresults.name_of_data_variable = zeros(1st_factor, 2nd_factor, ... , numsubs)

batchresults.total_trials_per_type = zeros(types_of_trials,numsubs); %A count of the total trials by type
batchresults.accuracy = zeros(types_of_trials,numsubs);%Was the participant's response accurate?

%EDITHERE Create the matrix of buckets for RT data:
batchresults.medianrt_cell = cell(types_of_trials, numsubs); 
batchresults.medianrt = zeros(types_of_trials, numsubs); 
batchresults.rt = zeros(types_of_trials ,numsubs);



%Subject counter, this is used because the subject number you will use will
%not always be ascending. This allows us to use any data file we wish
%regardless of the subject number.
subcount = 0;


%Loops through all of the subjects
for sub = 1:numsubs
    subcount = subcount + 1;  %Number of subjects so far
    
    %Load the data file corresponding to the current subject number
    fname = sprintf('%sExpSub_compact_%s.mat',datadir,subnum_list{sub});
    load(fname)
    
    %Block #1 Loop (There is only one block in this data file)
    block = 1;
    numtrials = length(Userdata.Blocks(block).Trials);
    
    %Loops through all of the trials in this block
    for(trial = 1:numtrials)
        
        %Extract crucial information from the data file for this trial
        
        trial_type = Userdata.Blocks(block).Block_Export.trialtype(trial);     %congruent, incongruent, neutral?
        whichword = Userdata.Blocks(block).Trials(trial).whichword;  %what specific word was shown?
        response = Userdata.Blocks(block).Trials(trial).response;    %what key was pressed?
        
        %EDITHERE Add code to extract the reaction time from the data file
        rt = Userdata.Blocks(block).Trials(trial).rt;
        
        
        %Define the condition levels
        %condition1 (trial_type)
        if trial_type == 1  %Congruent trial, sets condition1 to 1
            factor1  = 1;
        
        elseif trial_type == 2 %Incongruent trial
            factor1 = 2;
        
        elseif trial_type  == 3  %Neutral trial
            factor1 = 3;
        end
        
        %If you had more than one condition, you would specify the rest of
        %the levels here:
        
        
        %Keep count of how many trials there are of each condition
        %this line will add the value of 1 to the unit of the matrix that
        %matches the current condition and count (subject)
        batchresults.total_trials_per_type(factor1,subcount) = batchresults.total_trials_per_type(factor1,subcount) + 1;
        
        
        %Was the response accurate?
        %If the response is determined to be accurate, the value of the
        %variable "accurate" is set to 1 otherwise it is 0.
        accurate = 0;
        
        %Congruent trials
        if factor1 == 1
            if(response =='r' & whichword== 1)
                accurate = 1;
            end
            if(response =='g' & whichword== 2)
                accurate = 1;
            end
            if(response =='b' & whichword== 3)
                accurate = 1;
            end
            if(response =='y' & whichword== 4)
                accurate = 1;
            end
            if(response =='p' & whichword== 5)
                accurate = 1;
            end
        end
        
        %Incongruent
        if factor1 == 2
            if(response =='r' & (whichword== 5  || whichword== 10 || whichword== 14  || whichword == 17))
                accurate = 1;
            end
            if(response =='g' & (whichword== 1  || whichword== 9  || whichword== 13  || whichword == 18))
                accurate = 1;
            end
            if(response =='b' & (whichword== 2  || whichword== 7  || whichword== 15  || whichword == 19))
                accurate = 1;
            end
            if(response =='y' & (whichword== 3  || whichword== 6  || whichword== 11  || whichword == 20))
                accurate = 1;
            end
            if(response =='p' & (whichword== 4 || whichword== 8  || whichword== 12  || whichword == 16))
                accurate = 1;
            end
        end
        
        %Neutral
        if factor1 == 3 
            if(response =='b' & (whichword== 1 || whichword== 6  || whichword== 11 || whichword== 16))
                accurate = 1;
            end
            if(response =='r' &( whichword== 2 || whichword== 7  || whichword== 12 || whichword== 17))
                accurate = 1;
            end
            if(response =='g' & (whichword== 3 || whichword== 8  || whichword== 13 || whichword== 18))
                accurate = 1;
            end
            if(response =='y' & (whichword== 4 || whichword== 9  || whichword== 14 || whichword== 19))
                accurate = 1;
            end
            if(response =='p' & (whichword== 5 || whichword== 10 || whichword== 15 || whichword== 20))
                accurate = 1;
            end
        end
        
        %This line of code will add the value of "accurate" to the respective unit of the matrix
        %This will total the number of accurate trials.
        batchresults.accuracy(factor1,subcount) =  batchresults.accuracy(factor1,subcount) + accurate;
      
        %EDITHERE: Accumulate rt data just for all of the accurate trials here
        if accurate == 1%
        batchresults.rt(factor1,subcount) = batchresults.rt(factor1,subcount) + rt;
        %accumulate each of the individual RTs for a given condition and a
        
        batchresults.medianrt_cell{factor1,subcount} = cat(1, batchresults.medianrt_cell{factor1,subcount},rt);
       
        end
    end   
end

batchresults.subnum_list = subnum_list;

%EDITHERE Calculate the average reaction time here
batchresults.rt = batchresults.rt./batchresults.accuracy;

%EDITHERE Calculate the median reaction times here
 for factor1 = 1:3
    for subnum = 1:subcount
        batchresults.medianrt(factor1,subnum) = median(batchresults.medianrt_cell{factor1,subnum});
    end
end

%EDITHERE Calculate the mean of the median reaction times across subjects here
batchresults.meanofmedianrt = mean(batchresults.medianrt,2);


%EDITHERE Calculate the accuracy percentages here
batchresults.accuracy = batchresults.accuracy./batchresults.total_trials_per_type;





